@@ -161,7 +161,7 @@ static void InitGroups(Translator *tr)
 
 		if(p[0] == RULE_REPLACEMENTS)
 		{
-			pw = (unsigned int *)(((long)p+4) & ~3);  // advance to next word boundary
+			pw = (unsigned int *)(((long64)p+4) & ~3);  // advance to next word boundary
 			tr->langopts.replace_chars = pw;
 			while(pw[0] != 0)
 			{
@@ -524,27 +524,33 @@ unsigned short ipa1[96] = {
 };
 
 
-static void WritePhMnemonic(char **buf, PHONEME_TAB *ph, PHONEME_LIST *plist)
-{//=======================================================================================
+char *WritePhMnemonic(char *phon_out, PHONEME_TAB *ph, PHONEME_LIST *plist, int use_ipa)
+{//=====================================================================================
 	int c;
 	int mnem;
 	int len;
 	int first;
 	int ix = 0;
-	char *phon_out;
+	char *p;
 	unsigned int ipa_control=0;  // first byte of ipa string may control the phoneme name interpretation. 0x20 = ignore this phoneme
 	PHONEME_DATA phdata;
 
-	phon_out = *buf;
-
 	if(ph->code == phonEND_WORD)
 	{
 		// ignore
 		phon_out[0] = 0;
-		return;
+		return(phon_out);
 	}
 
-	if(option_phonemes == 3)
+	if(ph->code == phonSWITCH)
+	{
+		// the tone_ph field contains a phoneme table number
+		p = phoneme_tab_list[plist->tone_ph].name;
+		sprintf(phon_out, "(%s)", p);
+		return(phon_out + strlen(phon_out));
+	}
+
+	if(use_ipa)
 	{
 		// has an ipa name been defined for this phoneme ?
 		phdata.ipa_string[0] = 0;
@@ -568,8 +574,9 @@ static void WritePhMnemonic(char **buf, PHONEME_TAB *ph, PHONEME_LIST *plist)
 			}
 			if(ipa_control >= 0x20)
 			{
-				*buf += ix;
-				return;  // 0x20 = ignore phoneme
+				phon_out = &phon_out[ix];
+				*phon_out = 0;
+				return(phon_out);  // 0x20 = ignore phoneme
 			}
 		}
 	}
@@ -580,7 +587,7 @@ static void WritePhMnemonic(char **buf, PHONEME_TAB *ph, PHONEME_LIST *plist)
 		if((c == '/') && (option_phoneme_variants==0))
 			break;      // discard phoneme variant indicator
 
-		if(option_phonemes == 3)
+		if(use_ipa)
 		{
 			// convert from ascii to ipa
 			if(first && (c == '_'))
@@ -605,24 +612,25 @@ static void WritePhMnemonic(char **buf, PHONEME_TAB *ph, PHONEME_LIST *plist)
 		first = 0;
 	}
 
-	*buf += ix;
+	phon_out = &phon_out[ix];
+	*phon_out = 0;
+	return(phon_out);
 }  // end of WritePhMnemonic
 
 
 
-void GetTranslatedPhonemeString(char *phon_out, int n_phon_out)
-{//============================================================
+void GetTranslatedPhonemeString(char *phon_out, int n_phon_out, int use_ipa)
+{//=========================================================================
 /* Can be called after a clause has been translated into phonemes, in order
    to display the clause in phoneme mnemonic form.
 */
 
 	int  ix;
-	int  len;
-	int  max_len;
+	unsigned int  len;
+	unsigned int  max_len;
 	int  phon_out_ix=0;
 	int  stress;
 	unsigned int c;
-	char *p;
 	char *buf;
 	char phon_buf[30];
 	PHONEME_LIST *plist;
@@ -646,7 +654,7 @@ void GetTranslatedPhonemeString(char *phon_out, int n_phon_out)
 					c = 0;
 					if(stress > 5) stress = 5;
 
-					if(option_phonemes == 3)
+					if(use_ipa)
 					{
 						c = 0x2cc;  // ipa, secondary stress
 						if(stress > 3)
@@ -664,30 +672,22 @@ void GetTranslatedPhonemeString(char *phon_out, int n_phon_out)
 				}
 			}
 
-			if(plist->ph->code == phonSWITCH)
-			{
-				// the tone_ph field contains a phoneme table number
-				p = phoneme_tab_list[plist->tone_ph].name;
+			buf = WritePhMnemonic(buf, plist->ph, plist, use_ipa);
 
-				sprintf(buf, "(%s)", p);
-				buf += (strlen(p) + 2);
-			}
-			else
+			if(plist->ph->code != phonSWITCH)
 			{
-				WritePhMnemonic(&buf, plist->ph, plist);
-	
 				if(plist->synthflags & SFLAG_LENGTHEN)
 				{
-					WritePhMnemonic(&buf, phoneme_tab[phonLENGTHEN], NULL);
+					buf = WritePhMnemonic(buf, phoneme_tab[phonLENGTHEN], NULL, use_ipa);
 				}
 				if((plist->synthflags & SFLAG_SYLLABLE) && (plist->type != phVOWEL))
 				{
 					// syllablic consonant
-					WritePhMnemonic(&buf, phoneme_tab[phonSYLLABIC], NULL);
+					buf = WritePhMnemonic(buf, phoneme_tab[phonSYLLABIC], NULL, use_ipa);
 				}
 				if(plist->tone_ph > 0)
 				{
-					WritePhMnemonic(&buf, phoneme_tab[plist->tone_ph], NULL);
+					buf = WritePhMnemonic(buf, phoneme_tab[plist->tone_ph], NULL, use_ipa);
 				}
 			}
 	
@@ -776,7 +776,7 @@ static int IsLetter(Translator *tr, int letter, int group)
 
 	if(tr->letter_bits_offset > 0)
 	{
-		if(((letter2 = (letter - tr->letter_bits_offset)) > 0) && (letter2 < 0x80))
+		if(((letter2 = (letter - tr->letter_bits_offset)) > 0) && (letter2 < 0x100))
 				letter = letter2;
 		else
 			return(0);
@@ -787,7 +787,7 @@ static int IsLetter(Translator *tr, int letter, int group)
 			return(tr->letter_bits[remove_accent[letter-0xc0]] & (1L << group));
 	}
 
-	if((letter >= 0) && (letter < 0x80))
+	if((letter >= 0) && (letter < 0x100))
 		return(tr->letter_bits[letter] & (1L << group));
 
 	return(0);
@@ -832,7 +832,6 @@ int Unpronouncable(Translator *tr, char *word, int posn)
 	int  vowel_posn=9;
 	int  index;
 	int  count;
-	int  apostrophe=0;
 
 	utf8_in(&c,word);
 	if((tr->letter_bits_offset > 0) && (c < 0x241))
@@ -868,10 +867,7 @@ int Unpronouncable(Translator *tr, char *word, int posn)
 			break;
 		}
 
-		if(c == '\'')
-			apostrophe = 1;
-		else
-		if(!iswalpha(c))
+		if((c != '\'') && !iswalpha(c))
 			return(0);
 	}
 
@@ -1123,7 +1119,7 @@ void SetWordStress(Translator *tr, char *output, unsigned int *dictionary_flags,
 	int final_ph2;
 	int mnem;
 	int mnem2;
-	int post_tonic;
+	int post_tonic;   // currently not used
 	int opt_length;
 	int done;
 	int stressflags;
@@ -1387,7 +1383,7 @@ void SetWordStress(Translator *tr, char *output, unsigned int *dictionary_flags,
 		{
 			int wt;
 			int max_weight = -1;
-			int prev_stressed;
+//			int prev_stressed;
 
 			// find the heaviest syllable, excluding the final syllable
 			for(ix = 1; ix < (vowel_count-1); ix++)
@@ -1397,7 +1393,7 @@ void SetWordStress(Translator *tr, char *output, unsigned int *dictionary_flags,
 					if((wt = syllable_weight[ix]) >= max_weight)
 					{
 						max_weight = wt;
-						prev_stressed = stressed_syllable;
+//						prev_stressed = stressed_syllable;
 						stressed_syllable = ix;
 					}
 				}
@@ -1614,7 +1610,7 @@ void SetWordStress(Translator *tr, char *output, unsigned int *dictionary_flags,
 
 			if(v_stress <= 1)
 			{
-				if((v > 1) && (max_stress >= 4) && (stressflags & 4) && (v == (vowel_count-1)))
+				if((v > 1) && (max_stress >= 2) && (stressflags & 4) && (v == (vowel_count-1)))
 				{
 					// option: mark unstressed final syllable as diminished
 					v_stress = 0;
@@ -2161,9 +2157,19 @@ static void MatchRule(Translator *tr, char *word[], char *word_start, int group_
 					break;
 
 				case RULE_ENDING:
-					// next 3 bytes are a (non-zero) ending type. 2 bytes of flags + suffix length
-					match.end_type = (rule[0] << 16) + ((rule[1] & 0x7f) << 8) + (rule[2] & 0x7f);
-					rule += 3;
+					{
+						int end_type;
+						// next 3 bytes are a (non-zero) ending type. 2 bytes of flags + suffix length
+						end_type = (rule[0] << 16) + ((rule[1] & 0x7f) << 8) + (rule[2] & 0x7f);
+
+						if((tr->word_vowel_count == 0) && !(end_type & SUFX_P) && (tr->langopts.param[LOPT_SUFFIX] & 1))
+							failed = 1;	// don't match a suffix rule if there are no previous syllables (needed for lang=tr).
+						else
+						{
+							match.end_type = end_type;
+							rule += 3;
+						}
+					}
 					break;
 
 				case RULE_NO_SUFFIX:
@@ -2425,9 +2431,8 @@ int TranslateRules(Translator *tr, char *p_start, char *phonemes, int ph_size, c
    Append the result to 'phonemes' and any standard prefix/suffix in 'end_phonemes' */
 
 	unsigned char  c, c2;
-	unsigned int  c12, c123;
+	unsigned int  c12;
 	int wc=0;
-	int wc_prev;
 	int wc_bytes;
 	char *p2;           /* copy of p for use in double letter chain match */
 	int  found;
@@ -2465,7 +2470,7 @@ int TranslateRules(Translator *tr, char *p_start, char *phonemes, int ph_size, c
 	if((option_phonemes == 2) && ((word_flags & FLAG_NO_TRACE)==0))
 	{
 		char wordbuf[120];
-		int  ix;
+		unsigned int  ix;
 
 		for(ix=0; ((c = p_start[ix]) != ' ') && (c != 0) && (ix < (sizeof(wordbuf)-1)); ix++)
 		{
@@ -2487,7 +2492,6 @@ int TranslateRules(Translator *tr, char *p_start, char *phonemes, int ph_size, c
 	
 	while(((c = *p) != ' ') && (c != 0))
 	{
-		wc_prev = wc;
 		wc_bytes = utf8_in(&wc,p);
 		if(IsAlpha(wc))
 			any_alpha++;
@@ -2530,7 +2534,6 @@ int TranslateRules(Translator *tr, char *p_start, char *phonemes, int ph_size, c
 				/* there are some 2 byte chains for this initial letter */
 				c2 = p[1];
 				c12 = c + (c2 << 8);   /* 2 characters */
-				c123 = c12 + (p[2] << 16);
 	
 				g1 = tr->groups2_start[c];
 				for(g=g1; g < (g1+n); g++)
@@ -2570,7 +2573,6 @@ int TranslateRules(Translator *tr, char *p_start, char *phonemes, int ph_size, c
 					if((match1.points == 0) && ((option_sayas & 0x10) == 0))
 					{
 						n = utf8_in(&letter,p-1)-1;
-
 						if(tr->letter_bits_offset > 0)
 						{
 							// not a Latin alphabet, switch to the default Latin alphabet language
@@ -2928,7 +2930,7 @@ static const char *LookupDict2(Translator *tr, const char *word, const char *wor
 	const char *word_end;
 	const char *word1;
 	int wflags = 0;
-	int wflags2;
+//	int wflags2;
 	char word_buf[N_WORD_BYTES+1];
 
 	if(wtab != NULL)
@@ -3203,13 +3205,10 @@ static const char *LookupDict2(Translator *tr, const char *word, const char *wor
 		
 		if(option_phonemes == 2)
 		{
-			unsigned int flags1 = 0;
 			char ph_decoded[N_WORD_PHONEMES];
 			int textmode;
 
 			DecodePhonemes(phonetic,ph_decoded);
-			if(flags != NULL)
-				flags1 = flags[0];
 
 			if((dictionary_flags & FLAG_TEXTMODE) == 0)
 				textmode = 0;
@@ -3497,6 +3496,17 @@ int RemoveEnding(Translator *tr, char *word, int end_type, char *word_copy)
 	
 	if(end_type & SUFX_E)
 	{
+		if(tr->translator_name == L('n','l'))
+		{
+			if(((word_end[0] & 0x80) == 0) && ((word_end[-1] & 0x80) == 0) && IsVowel(tr, word_end[-1]) && IsLetter(tr, word_end[0], LETTERGP_C) && !IsVowel(tr, word_end[-2]))
+			{
+				//double the vowel before the (ascii) final consonant
+				word_end[1] = word_end[0];
+				word_end[0] = word_end[-1];
+				word_end[2] = ' ';
+			}
+		}
+		else
 		if(tr->translator_name == L('e','n'))
 		{
 			// add 'e' to end of stem
